'''
Created on Dec. 3, 2013

Contains a test for the dynamic programming package

@author: Andre Dozier
'''

import dypro.dypro as dp 
import numpy as np
# import matplotlib.pyplot as plt

testname = 'Bertsekas (2005) Ex. 1.3.2'
def run(verbose=False, doplots=False): 
    # test from Bertsekas (2005). Dynamic Programming and Optimal Control. pp.28-32
    
    # set up random variable
    w = np.array([[0], [1], [2]])
    p_w = np.array([0.1, 0.7, 0.2])
    
    # set up the state and decision grids and time   
    t = range(4)
    stategrid = dp.LinGrid(0, 2, 3)
    decisiongrid = dp.LinGrid(0, 2, 3)
    
    def state1_3_2(x, u, t, w):
        # defines the state equation
        val = x + u - w
        if val < 0: 
            return np.zeros(1)
        return val 
    
    def sim1_3_2(x, u, t, w): 
        return (state1_3_2(x, u, t, w), u)
    
    def value1_3_2(x, xnext, u, t, w): 
        # defines the value function 
        if x + u > 2:
            return np.Inf
        return u + (x + u - w) ** 2
    
    def transProb(x, nextx, u, t, rndvar_i): 
        # defines the transition probabilities function 
        return p_w[rndvar_i]
    
    # solve the problem  
    opt_results = dp.solve_finitetime(
                                value1_3_2, state1_3_2, stategrid, decisiongrid,
                                t, ismax=False, rndvar_space=w, 
                                transitionP=transProb, verbose=verbose)

    # show all values for every state
    if verbose:
        print 'all values for all states at each stage'
        print opt_results['opt_values']
        print 'all decision indices'
        print opt_results['opt_decisions_indices']
        print 'all next states'
        print opt_results['opt_next_states_indices']
    
    # trace optimal paths
#     decisions, _, values = \
#          dp.trace_optimal_paths(stategrid, decisiongrid, t, np.array([0]),
#                     opt_decisions_indices, opt_next_states_indices,
#                     opt_values, ismax=False, verbose=verbose)            
    trace_results = dp.trace_optimal_paths(stategrid, decisiongrid, t, np.array([0]),
                    opt_results, ismax=False, verbose=False, 
                    rndvar_space=w, transitionP=transProb, valuefxn=value1_3_2, 
                    simfxn=sim1_3_2, nscenarios=1000)            
                    
    # extract desired information             
    decisions = trace_results['trace_decisions']
    states = trace_results['trace_states']
    values = trace_results['trace_values']
    if 'trace_rndvars' in trace_results: 
        rndvars = trace_results['trace_rndvars']
    
    # # plot traces
    # if doplots:
        # # fix variables to be 2D arrays (each column is a different stochastically generated timeseries)  
        # states = np.array([s.reshape((-1,)) for s in states])
        # decisions = np.array([d.reshape((-1,)) for d in decisions])
        
        # # plot states 
        # plt.figure(1)  
        # plt.clf()
        # plt.plot(t, states)
        # plt.xlabel('time')
        # plt.ylabel('state')
        # if verbose: 
            # print 'mean states:'
            # print states.mean(axis=1)
        
        # # plot optimal decisions 
        # plt.figure(2)
        # plt.clf()
        # plt.plot(t[:-1], decisions)
        # plt.xlabel('time')
        # plt.ylabel('decision')
        # if verbose: 
            # print 'mean decisions:'
            # print decisions.mean(axis=1)
        
        # # show plot
        # plt.ion() 
        # plt.show()

    # check optimal decisions
    correct = (np.abs(np.array([d.reshape((-1,)) for d in opt_results['opt_decisions']]) - np.array([[0,0,1], [0,0,1], [0,0,1]])) < 0.0001)
    if not correct.all(): 
        return False
    
    # check optimal values
    correct = (np.abs(opt_results['opt_values'] - np.array([[2.818, 2.7, 3.7],[1.68, 1.5, 2.5],[1.1, 0.3, 1.3],[0., 0., 0.]])) < 0.0001)
    if not correct.all(): 
        return False

    return True

if __name__ == '__main__':
    if run(verbose=True, doplots=True): 
        print 'passed'
    else: 
        print 'failed'