'''
Created on Dec. 3, 2013

Contains a test for the dynamic programming package

@author: Andre Dozier
'''

import dypro.dypro as dp 
import numpy as np 
# import matplotlib.pyplot as plt

testname = 'Miranda and Fackler (2002) Ex. 7.6.1'
def run(verbose=False, doplots=False): 
    # test from Miranda and Fackler (2002). Applied Computational Economics and Finance. p. 172
    
    # set up model parameters 
    price = 1
    sbar = 100 
    delta = 0.9 
    state0 = np.array([sbar])
    n = sbar + 1  

    # set up the state and decision grids and time   
    stategrid = dp.LinGrid(0, sbar, n)
    decisiongrid = dp.LinGrid(0, sbar, n)
    
    # time 
    t = range(15)

    # maximize or minimize? 
    ismax = True  

    # define the state equation
    def statefxn(s, x, t, w):
        return s - x
    
    # define the value function 
    def valuefxn(s, snext, x, t, w):
        if x <= s:  
            return price * x - x ** 2 / (1 + s) 
        else: 
            return -np.Inf
    
    # solve the problem  
    opt_results = dp.solve_finitetime(valuefxn, statefxn, stategrid, decisiongrid, t, ismax=ismax, discount=delta, verbose=verbose)

    # show all values for every state
    if verbose: 
        # print 'all values for all states at each stage'
        # print opt_values
        print 'optimal decision indices'
        print opt_results['opt_decisions_indices']
     
    # trace optimal paths
    trace_results = dp.trace_optimal_paths(stategrid, decisiongrid, t, state0,
                    opt_results, ismax=ismax, verbose=verbose)            
                    
    # extract desired information             
    decisions = trace_results['trace_decisions']
    states = trace_results['trace_states']
    values = trace_results['trace_values']

    # if doplots: 
        # plt.ion()
        # plt.close('all')
        
        # # plot decisions against time 
        # plt.figure()
        # timearray = np.array(t[:-1]).reshape((-1,1))
        # print timearray.shape
        # print decisions.shape
        # plt.plot(timearray, decisions, 'k-')
        # plt.xlabel('Time')
        # plt.ylabel('Optimal extraction path')
        
        # # plot stock against time 
        # timearray = np.array(t).reshape((-1,1))
        # plt.figure()
        # plt.plot(timearray, states, 'k-')
        # plt.xlabel('Time')
        # plt.ylabel('Optimal stock depletion path')
        
        # # plot stock against time 
        # plt.figure()
        # plt.plot(timearray, values, 'k-')
        # plt.xlabel('Time')
        # plt.ylabel('Optimal value path')

        # plt.show() 

    # check optimal decisions
    correct_decisions = np.array([24.,18.,14.,11.,8.,6.,5.,4.,3.,2.,2.,1.,1.,1.]).reshape((-1, 1))
    correct = (np.abs(decisions - correct_decisions) < 0.0001)
    if not correct.all(): 
        return False
     
    # check optimal values
    correct_states = np.array([100.,76.,58.,44.,33.,25.,19.,14.,10.,7.,5.,3.,2.,1.,0.]).reshape((-1, 1))
    correct = (np.abs(states - correct_states) < 0.0001)
    if not correct.all(): 
        return False

    return True

if __name__ == '__main__':
    if run(verbose=True, doplots=True): 
        print '\n\nPassed test!!!'
    else: 
        print '\n\nFailed test...'
    