'''
Created on Dec. 3, 2013

Contains a test for the dynamic programming package

@author: Andre Dozier
'''

import dypro.dypro as dp 
import numpy as np 
# import matplotlib.pyplot as plt

testname = 'Miranda and Fackler (2002) Ex. 7.6.2'
def run(verbose=False, doplots=False): 
    # test from Miranda and Fackler (2002). Applied Computational Economics and Finance. p. 175
    
    # set up model parameters 
    maxage = 5
    repcost = 75.
    delta = 0.9
    n = maxage
    state0 = np.array([1])
     
    # set up the state and decision grids and time   
    stategrid = dp.LinGrid(1, maxage, n)
    decisiongrid = dp.LinGrid(0, 1, 2) # binary variable (keep = 0, replace = 1)
    
    # time 
    t = range(12)

    # maximize or minimize? 
    ismax = True

    # define the state equation
    def statefxn(s, x, t, w):
        if x == 0: # keep
            return min(s + 1, n)
        else: # replace
            return 1
    
    # define the value function 
    def valuefxn(s, snext, x, t, w):
        if x == 0: # keep
            return 50 - 2.5 * s - 2.5 * (s ** 2)
        else: # replace
            return (50 - repcost)
    
    # solve the problem  
    opt_results = dp.solve_finitetime(valuefxn, statefxn, stategrid, decisiongrid, t, ismax=ismax, discount=delta, verbose=verbose)
     
    # trace optimal paths
    trace_results = dp.trace_optimal_paths(stategrid, decisiongrid, t, state0,
                    opt_results, ismax=ismax, verbose=verbose)            
                    
    # extract desired information             
    decisions = trace_results['trace_decisions']
    states = trace_results['trace_states']
    values = trace_results['trace_values']

    # if doplots: 
        # plt.ion()
        # plt.close('all')
        
        # # plot decisions against time 
        # plt.figure()
        # timearray = np.array(t[:-1]).reshape((-1,1))
        # plt.plot(timearray, decisions, 'k-')
        # plt.xlabel('Time')
        # plt.ylabel('Optimal decision path')
        
        # # plot stock against time 
        # timearray = np.array(t).reshape((-1,1))
        # plt.figure()
        # plt.plot(timearray, states, 'k-')
        # plt.xlabel('Time')
        # plt.ylabel('Optimal state path')
        
        # # plot stock against time 
        # plt.figure()
        # plt.plot(timearray, values, 'k-')
        # plt.xlabel('Time')
        # plt.ylabel('Optimal value path')

        # plt.show() 

    # check optimal decisions
    correct_decisions = np.array([0,0,0,1,0,0,0,1,0,0,0]).reshape((-1, 1))
    correct = (np.abs(decisions - correct_decisions) < 0.0001)
    if not correct.all(): 
        return False
        
    # check optimal values
    correct_states = np.array([1,2,3,4,1,2,3,4,1,2,3,4]).reshape((-1, 1))
    correct = (np.abs(states - correct_states) < 0.0001)
    if not correct.all(): 
        return False
 
    return True

if __name__ == '__main__':
    if run(verbose=True, doplots=True): 
        print '\n\nPassed test!!!'
    else: 
        print '\n\nFailed test...'
    