'''
Created on April 21, 2014

@author: Andre Dozier
'''

import matplotlib.pyplot as plt
import fnmatch
import os
import numpy as np

# define a function to list files within all subdirectories
def myglob(thedir, thepattern):
    matches = []
    for root, dirnames, filenames in os.walk(thedir):
      for filename in fnmatch.filter(filenames, thepattern):
          matches.append(os.path.join(root, filename))
    return matches

# get all the csv files with present values in them
thisdir = os.path.abspath(os.path.dirname(__file__))
figdir = os.path.join(thisdir, 'figures_annual')
origdir = os.path.join(figdir, 'Original') 

# clear other plots 
plt.close('all')


#################################
#####  FUNCTION DEFINITIONS 
#################################
def plot_present_value(pvals, savename, ylabel, firstlabel=None, secondvals=None, secondlabel=None, thirdvals=None, thirdlabel=None, t=0):
    # get all the values in a list 
    allvals = [np.load(f)[:,:,0].flatten() for f in pvals]
    allvals = [v[np.isfinite(v)] for v in allvals]
    allnames = [os.path.basename(os.path.dirname(f)) for f in pvals] 
    
    # plot
    fig = plt.figure(figsize=(15,7.5))
    ax = fig.add_subplot(111) 
    if thirdvals != None: 
        thirdvals = [np.load(f)[:,:,0].flatten() for f in thirdvals]
        plt.boxplot(thirdvals, positions=list(np.arange(len(thirdvals)) + 1.6))
    plt.boxplot(allvals)
    if secondvals != None: 
        secondvals = [np.load(f)[:,:,0].flatten() for f in secondvals]
        plt.boxplot(secondvals, positions=list(np.arange(len(secondvals)) + 1.3))
    
    # change the xticks
    locs, labels = plt.xticks()
    newlabels = []
    for i in range(len(allnames)):
        newlabel = allnames[i]  
        if firstlabel != None: 
            newlabel += firstlabel[i]
        if secondlabel != None: 
            newlabel += secondlabel[i] 
        if thirdlabel != None:
            newlabel += thirdlabel[i]
        newlabels.append(newlabel) 
    # ax.set_xticklabels(newlabels, rotation=45)
    plt.xticks(locs, newlabels)
    
    # labels
    plt.xlim([0.5, len(allvals) + 1])
    plt.xlabel('Scenario',fontsize=14)
    plt.ylabel(ylabel,fontsize=14)
    
    # save 
    plt.tight_layout()
    plt.savefig(os.path.join(figdir, savename), dpi=300)


#################################
#####  ORIGINAL SCENARIO 
#################################
pvals = myglob(origdir, '*_trace_values.npy')
plot_present_value(pvals, 'original.png', 'Present Value ($)') 

#################################
#####  NO STORAGE SCENARIO 
#################################
pvals = myglob(origdir, '*_trace_nostorvalues.npy')
plot_present_value(pvals, 'original_nostor.png', 'Present Value without storage ($)')

#################################
#####  COMPARE NO STORAGE WITH  
#################################
marg = 'marg'
# get original and mean shift
ovals = myglob(os.path.join(figdir, 'Original/FreeMarket'), '*_trace_%svalues.npy' % marg)
ovals += myglob(os.path.join(figdir, 'Original/Ag85Muni15'), '*_trace_%svalues.npy' % marg)
ovals += myglob(os.path.join(figdir, 'Mean10/FreeMarket'), '*_trace_%svalues.npy' % marg)
ovals += myglob(os.path.join(figdir, 'Mean10/Ag85Muni15'), '*_trace_%svalues.npy' % marg)

# get no storage scenarios
svals = myglob(os.path.join(figdir, 'Stor-100.0/FreeMarket'), '*_trace_%svalues.npy' % marg)
svals += myglob(os.path.join(figdir, 'Stor-100.0/Ag85Muni15'), '*_trace_%svalues.npy' % marg)
svals += myglob(os.path.join(figdir, 'Stor-100.0Mean10/FreeMarket'), '*_trace_%svalues.npy' % marg)
svals += myglob(os.path.join(figdir, 'Stor-100.0Mean10/Ag85Muni15'), '*_trace_%svalues.npy' % marg)

# get additional storage and mean shift 
pvals = myglob(os.path.join(figdir, 'Stor+12.5/FreeMarket'), '*_trace_%svalues.npy' % marg)
pvals += myglob(os.path.join(figdir, 'Stor+12.5/Ag85Muni15'), '*_trace_%svalues.npy' % marg)
pvals += myglob(os.path.join(figdir, 'Stor+12.5Mean10/FreeMarket'), '*_trace_%svalues.npy' % marg)
pvals += myglob(os.path.join(figdir, 'Stor+12.5Mean10/Ag85Muni15'), '*_trace_%svalues.npy' % marg)

# plot present values
plot_present_value(ovals, 'storage.png', 'Present Value ($)',
                   secondvals = svals, thirdvals = pvals, 
                   firstlabel=[' 100% '] * 2 + [' 90% '] * 2, 
                   secondlabel=['(nostor,'] * 4,
                   thirdlabel=['12.5stor)'] * 4) 



# show the plots 
plt.ion()
plt.show()
